##
# LazyString is a class for computing strings only when necessary -- that is,
# when #to_str or #to_s is called. This is useful when computing the string is
# expensive, but the string is unlikely to be used: an error message passed as
# a method argument, for example.
#
class LazyString
  ##
  # The version string.
  #
  VERSION = '0.3.0'

  ##
  # Creates a new lazy string. The block argument will be saved for later use
  # by #to_str. If no block argument is given, it will default to a block that
  # returns an empty string.
  #
  def initialize(&block)
    @block = block || lambda { '' }
  end

  ##
  # A synonym for #to_str.
  #
  def to_s
    to_str
  end

  ##
  # Returns the computed string. It will compute the string by calling (with no
  # arguments) the block passed to ::new. If the block's return value responds
  # to the \to_str message, \to_str will be called on it to create the string.
  # Otherwise, \to_s will be called on it. The computed string will be saved,
  # so subsequent calls of this method will not cause the block to be called
  # again.
  #
  def to_str
    @str ||= begin
      result = @block.call
      if result.respond_to?(:to_str)
        result.to_str
      else
        result.to_s
      end
    end
  end
end
